<?php

/**
 * @file
 * Contains \Drupal\fieldblock\Plugin\Derivative\FieldBlock.
 */

namespace Drupal\fieldblock\Plugin\Derivative;

use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\Entity\Entity\EntityViewDisplay;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Provides block plugin definitions for fieldblock blocks.
 *
 * @see \Drupal\fieldblock\Plugin\Block\FieldBlock
 */
class FieldBlock extends DeriverBase {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    $blocks = FieldBlockHelper::fieldBlockGetBlockList();

    foreach ($blocks as $fieldblock_id => $description) {
      $this->derivatives[$fieldblock_id] = $base_plugin_definition;
      $this->derivatives[$fieldblock_id]['admin_label'] = $description;
    }

    return $this->derivatives;
  }

  /**
   * Builds a list of fields that have been made available as a block.
   *
   * @return array An array of fieldblocks.
   */
  public static function fieldBlockGetBlockList() {
    $fieldblocks = &drupal_static(__FUNCTION__);

    if (!isset($fieldblocks)) {
      $fieldblocks = array();

      // Get all EntityViewDisplay config entities and iterate over them.
      $entity_view_displays = EntityViewDisplay::loadMultiple();
      /** @var \Drupal\Core\Entity\EntityDisplayModeInterface $entity_view_display */
      foreach ($entity_view_displays as $display_id => $entity_view_display) {
        $view_display_fieldblocks = $entity_view_display->getThirdPartySettings('fieldblock');
        $entity_type = $entity_view_display->get('targetEntityType');
        $bundle = $entity_view_display->get('bundle');
        $mode = $entity_view_display->get('mode');

        foreach ($view_display_fieldblocks as $field_name => $field_label) {
          $fieldblock_id = $display_id . ':' . $field_name;
          $fieldblocks[$fieldblock_id] = t('@field field (from @type: @bundle: @mode)', array(
            '@field' => $field_label,
            '@type' => $entity_type,
            '@bundle' => $bundle,
            '@mode' => $mode,
          ));
        }
      }
    }

    return $fieldblocks;
  }

}
